/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.annotatedlineeditor;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.reconciler.DirtyRegion;
import org.eclipse.jface.text.reconciler.IReconcilingStrategy;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.ISourceViewer;


/**
 * Doesn't work!
 *
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class AnnotatedLineEditorReconcilingStrategy implements IReconcilingStrategy {
	private static final Logger logger = Logger
			.getLogger(AnnotatedLineEditorReconcilingStrategy.class);

	private IDocument document;
	private ISourceViewer viewer;

	/**
	 * <p>Constructor for AnnotatedLineEditorReconcilingStrategy.</p>
	 *
	 * @param sourceViewer a {@link org.eclipse.jface.text.source.ISourceViewer} object.
	 */
	public AnnotatedLineEditorReconcilingStrategy(ISourceViewer sourceViewer) {
		this.viewer = sourceViewer;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.reconciler.IReconcilingStrategy#reconcile(org.eclipse.jface.text.IRegion)
	 */
	/** {@inheritDoc} */
	public void reconcile(IRegion partition) {
		// not used
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.reconciler.IReconcilingStrategy#reconcile(org.eclipse.jface.text.reconciler.DirtyRegion, org.eclipse.jface.text.IRegion)
	 */
	/** {@inheritDoc} */
	public void reconcile(DirtyRegion dirtyRegion, IRegion subRegion) {
		try {
			doReconcile(dirtyRegion);
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	synchronized private void doReconcile(DirtyRegion dirtyRegion) throws BadLocationException {
		// Get all affected lines
		int dirtyOffset = dirtyRegion.getOffset();
		int dirtyLength = dirtyRegion.getLength();
		
		int lineChangeStart = document.getLineOfOffset(dirtyOffset);
		int lineChangeEnd;
		if (DirtyRegion.INSERT.equals(dirtyRegion.getType())) {
			lineChangeEnd = document.getLineOfOffset(dirtyOffset + dirtyLength);
		} else {
			lineChangeEnd = lineChangeStart;
		}
		
		for (int i = lineChangeStart; i <= lineChangeEnd; i++) {
			int lineOffset = document.getLineOffset(i);
			int lineLength = document.getLineLength(i);
			IRegion lineRegion = document.getLineInformation(i);
			String lineText = document.get(lineRegion.getOffset(), lineRegion.getLength());
			
			List<LineAnnotation> annotationsCurrentLine = 
					((LineAnnotationModel) getAnnotationModel()).getUndeletedAnnotations(lineOffset, lineLength);
			
			if (annotationsCurrentLine.size() == 0) {
				logger.debug("Adding new annotation " + lineText);
				addAnnotation(lineText, lineRegion);
			} else {
				LineAnnotation currentAnnotation = annotationsCurrentLine.get(0);
				int length = lineLength;
				getAnnotationModel().getPosition(currentAnnotation).setLength(length);
				currentAnnotation.setText(lineText);
				
				for (int j = 1; j < annotationsCurrentLine.size(); j++) {
					annotationsCurrentLine.get(j).markAsMerged(currentAnnotation.getEntity());
					getAnnotationModel().removeAnnotation(annotationsCurrentLine.get(j));
				}
			}
		}
		
		// Deletes have already taken place
		
		((LineAnnotationModel) getAnnotationModel()).printAnnotations();
	}
		
	private void addAnnotation(String text, IRegion region) {
		Position position = new Position(region.getOffset(), region.getLength());
		Annotation annotation = ((LineAnnotationModel) getAnnotationModel()).createAnnotation(text);
		getAnnotationModel().addAnnotation(annotation, position);
		
//		Position position2 = new Position(region.getOffset() + 1, region.getLength() - 2);
//		Annotation annotation2 = new Annotation("org.eclipse.ui.workbench.texteditor.error",true,"asdf");
//		getAnnotationModel().addAnnotation(annotation2, position2);
	}

	private IAnnotationModel getAnnotationModel() {
		return viewer.getAnnotationModel();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.reconciler.IReconcilingStrategy#setDocument(org.eclipse.jface.text.IDocument)
	 */
	/** {@inheritDoc} */
	public void setDocument(IDocument document) {
		this.document = document;		
	}
}
