/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.annotatedlineeditor;

import java.util.Iterator;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.DefaultInformationControl;
import org.eclipse.jface.text.DefaultTextHover;
import org.eclipse.jface.text.IInformationControl;
import org.eclipse.jface.text.IInformationControlCreator;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.ITextHoverExtension;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.reconciler.IReconciler;
import org.eclipse.jface.text.reconciler.IReconcilingStrategy;
import org.eclipse.jface.text.reconciler.MonoReconciler;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.editors.text.EditorsUI;


/**
 * <p>AnnotatedLineEditorSourceViewerConfiguration class.</p>
 *
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class AnnotatedLineEditorSourceViewerConfiguration extends
		SourceViewerConfiguration {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger.getLogger(AnnotatedLineEditorSourceViewerConfiguration.class);
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getReconciler(org.eclipse.jface.text.source.ISourceViewer)
	 */
	/** {@inheritDoc} */
	public IReconciler getReconciler(ISourceViewer sourceViewer) {
		
		IReconcilingStrategy strategy = new AnnotatedLineEditorReconcilingStrategy(sourceViewer);
		IReconciler reconciler = new MonoReconciler(strategy, true);

		return reconciler;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getTextHover(org.eclipse.jface.text.source.ISourceViewer, java.lang.String, int)
	 */
	/** {@inheritDoc} */
	@Override
	public ITextHover getTextHover(ISourceViewer sourceViewer, String contentType) {
		/**
		 * http://dev.eclipse.org/newslists/news.eclipse.platform/msg76821.html
		 * 
		 * The Javadoc hover is shown in a Browser widget
		 */
		return new TextHover(sourceViewer);
	}
	
	private final class TextHover extends DefaultTextHover implements ITextHoverExtension {

		private ISourceViewer sourceViewer;

		public TextHover(ISourceViewer sourceViewer) {
			super(sourceViewer);
			
			this.sourceViewer = sourceViewer;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.jface.text.DefaultTextHover#getHoverInfo(org.eclipse.jface.text.ITextViewer, org.eclipse.jface.text.IRegion)
		 */
		@Override
		public String getHoverInfo(ITextViewer textViewer, IRegion hoverRegion) {
			
			IAnnotationModel model= sourceViewer.getAnnotationModel();
			Iterator e= model.getAnnotationIterator();
			while (e.hasNext()) {
				Annotation annotation= (Annotation) e.next();
				if (isIncluded(annotation)) {
					Position p= model.getPosition(annotation);
					if (p != null && p.overlapsWith(hoverRegion.getOffset(), hoverRegion.getLength())) {
						return ((LineAnnotation) annotation).getHoverText();
					}
				}
			}
			return null;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.jface.text.ITextHoverExtension#getHoverControlCreator()
		 */
		public IInformationControlCreator getHoverControlCreator() {
			return new IInformationControlCreator() {
				public IInformationControl createInformationControl(Shell parent) {
					return new DefaultInformationControl(parent, EditorsUI.getTooltipAffordanceString());
				}
			};
		}
	}
}
