/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.annotatedlineeditor;

import java.util.Set;

/**
 * <p>IEntityContainer interface.</p>
 *
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 * @param <T>
 */
public interface IEntityContainer<T> {

	/**
	 * <p>getEntity</p>
	 *
	 * @param <T> a T object.
	 * @return a T object.
	 */
	T getEntity();

	void setEntity(Object entity);

	/**
	 * <p>getEditableText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	String getEditableText();

	/**
	 * <p>getAttachedEntities</p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	Set<T> getAttachedEntities();

	/**
	 * <p>markAsMerged</p>
	 *
	 * @param mergeTarget a T object.
	 */
	void markAsMerged(T mergeTarget);

	/**
	 * <p>markAsDeleted</p>
	 */
	void markAsDeleted();

	/**
	 * <p>markAsNew</p>
	 *
	 * @param isNew a boolean.
	 */
	void markAsNew(boolean isNew);

	/**
	 * <p>isMarkedAsMerged</p>
	 *
	 * @return a boolean.
	 */
	boolean isMarkedAsMerged();

	/**
	 * <p>isMarkedAsDeleted</p>
	 *
	 * @return a boolean.
	 */
	boolean isMarkedAsDeleted();

	/**
	 * <p>isMarkedAsNew</p>
	 *
	 * @return a boolean.
	 */
	boolean isMarkedAsNew();

	/**
	 * <p>setText</p>
	 *
	 * @param text a {@link java.lang.String} object.
	 */
	void setText(String text);

	/**
	 * <p>getMergeTarget</p>
	 *
	 * @return a T object.
	 */
	T getMergeTarget();

	/**
	 * <p>isDirty</p>
	 *
	 * @return a boolean.
	 */
	boolean isDirty();

	/**
	 * <p>setDirty</p>
	 *
	 * @param dirty a boolean.
	 */
	void setDirty(boolean dirty);
}
