/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.annotatedlineeditor;

import java.util.Map;

import eu.etaxonomy.cdm.model.common.ICdmBase;


/**
 * <p>IEntityCreator interface.</p>
 *
 * @author p.ciardelli
 * @created 03.07.2009
 * @version 1.0
 */
public interface IEntityCreator<T> {
	
	/**
	 * <p>createEntity</p>
	 *
	 * @param text a {@link java.lang.String} object.
	 * @param <T> a T object.
	 * @return a T object.
	 */
	T createEntity(String text);
	
	/**
	 * <p>createEntity</p>
	 *
	 * @param key a {@link java.lang.Object} object.
	 * @param text a {@link java.lang.String} object.
	 * @return a T object.
	 */
	T createEntity(Object key, String text);
	
	/**
	 * Get a map of keys for all objects that can be created by this entity creator
	 * along with their string representations.
	 *
	 * @return a {@link java.util.Map} object.
	 */
	Map<Object, String> getKeyLabelPairs();
	
	/**
	 * Return a flag indicating whether this creator also saves the entities
	 *
	 * @return true or false
	 */
	boolean savesEntity();
	

}
