/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.annotatedlineeditor;

import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;

/**
 * Methods for translating between the object in a line and
 * its display in the editor.
 *
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public interface ILineDisplayStrategy {

	/** Constant <code>SUPPLEMENTAL_TEXT="supplemental_text"</code> */
	public static final String SUPPLEMENTAL_TEXT = "supplemental_text";
	
	/**
	 * <p>getIndent</p>
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @return a int.
	 */
	int getIndent(Object entity);
	
	/**
	 * <p>getIcon</p>
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @return a {@link org.eclipse.swt.graphics.Image} object.
	 */
	Image getIcon(Object entity);
	
	/**
	 * <p>getFont</p>
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @return a {@link org.eclipse.swt.graphics.Font} object.
	 */
	Font getFont(Object entity);
	
	/**
	 * For object's title cache output, mainly
	 *
	 * Note: if object's cache is empty, return "", not null
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @return a {@link java.lang.String} object.
	 */
	String getText(Object entity);
	
	/**
	 * For object's supplemental information such as type, markers, etc.,
	 * usually stored in " [...]".
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @return a {@link java.lang.String} object.
	 */
	String getSupplementalText(Object entity);
	
	/**
	 * <p>getEmptyCacheMessage</p>
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @return a {@link java.lang.String} object.
	 */
	String getEmptyCacheMessage(Object entity);
	
	/**
	 * <p>setText</p>
	 *
	 * @param text a {@link java.lang.String} object.
	 * @param entity a {@link java.lang.Object} object.
	 */
	void setText(String text, Object entity);
		
	/**
	 * <p>getHoverText</p>
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @return a {@link java.lang.String} object.
	 */
	String getHoverText(Object entity);
	
	/**
	 * <p>setStatusMessage</p>
	 *
	 * @param text a {@link java.lang.String} object.
	 * @param entity a {@link java.lang.Object} object.
	 */
	void setStatusMessage(String text, Object entity);
	
	/**
	 * <p>isEntityCacheEmpty</p>
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @return a boolean.
	 */
	boolean isEntityCacheEmpty(Object entity);

	/**
	 * <p>addDisplayListener</p>
	 *
	 * @param entity a {@link java.lang.Object} object.
	 * @param listener a {@link eu.etaxonomy.taxeditor.annotatedlineeditor.EntityListener} object.
	 */
	void addDisplayListener(Object entity, EntityListener listener);
}
