/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.annotatedlineeditor;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.text.source.IOverviewRuler;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.taxeditor.model.LineSelection;

/**
 * Notifies selection service every time the user changes focus to a new line.
 *
 * @author p.ciardelli
 * @created 03.07.2009
 * @version 1.0
 */
public class LineSelectionViewer extends SourceViewer {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(LineSelectionViewer.class);

	private int lastLine = 0;
	private ILineSelectionSource lineSelectionSource;
	
	/**
	 * <p>Constructor for LineSelectionViewer.</p>
	 *
	 * @param parent a {@link org.eclipse.swt.widgets.Composite} object.
	 * @param verticalRuler a {@link org.eclipse.jface.text.source.IVerticalRuler} object.
	 * @param overviewRuler a {@link org.eclipse.jface.text.source.IOverviewRuler} object.
	 * @param showAnnotationsOverview a boolean.
	 * @param styles a int.
	 */
	public LineSelectionViewer(Composite parent, IVerticalRuler verticalRuler,
			IOverviewRuler overviewRuler, boolean showAnnotationsOverview,
			int styles) {
		super(parent, verticalRuler, overviewRuler, showAnnotationsOverview, styles);

//		createLineChangeListeners();
		setLineSelectionSource(new LineSelectionSource(this));
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.TextViewer#getSelection()
	 */
	/** {@inheritDoc} */
	@Override
	public ISelection getSelection() {
		ISelection selection = super.getSelection();
		if (getDocument() != null) {
			selection = createLineSelection((TextSelection) selection);
		}
		return selection;
	}
	
	/**
	 * <p>Setter for the field <code>lineSelectionSource</code>.</p>
	 *
	 * @param lineSelectionSource a {@link eu.etaxonomy.taxeditor.annotatedlineeditor.ILineSelectionSource} object.
	 */
	public void setLineSelectionSource(ILineSelectionSource lineSelectionSource) {
		this.lineSelectionSource = lineSelectionSource;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.TextViewer#firePostSelectionChanged(int, int)
	 */
	/** {@inheritDoc} */
	@Override
	protected void firePostSelectionChanged(int offset, int length) {
		if (redraws()) {
			IRegion r= widgetRange2ModelRange(new Region(offset, length));
			ITextSelection textSelection= r != null ? new TextSelection(getDocument(), r.getOffset(), r.getLength()) : TextSelection.emptySelection();
			LineSelection lineSelection = createLineSelection(textSelection);
			SelectionChangedEvent event= new SelectionChangedEvent(this, lineSelection);
			fireSelectionChanged(event);
		}
	}
	
	/**
	 * Creates a new instance of a LineSelection based on a given ITextSelection
	 * 
	 * @param textSelection
	 * @return
	 */
	private LineSelection createLineSelection(ITextSelection textSelection) {
		if (textSelection instanceof LineSelection ) {
			return (LineSelection) textSelection;
		}
		int line = textSelection.getStartLine();
		Object selectionObject = lineSelectionSource.getSelectedObjectAtLine(line);
		return new LineSelection(textSelection, getDocument(), selectionObject);
	}
}
