/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

/**
 * @author n.hoffmann
 * @created Dec 14, 2010
 * @version 1.0
 */
public class BulkEditorSortCombo {

	private Label label;
	
	private Combo combo;
	
	private List<IBulkEditorSortProvider> sortProviders;
		
	private List<Comparator> comparators = new ArrayList<Comparator>();
	
	private int selectedIndex = 0;
	
	/**
	 * 
	 */
	public BulkEditorSortCombo(Composite parent, List<IBulkEditorSortProvider> sortProviders) {
		this.sortProviders = sortProviders;
		if(! sortProviders.isEmpty()){
			label = new Label(parent, SWT.NONE);
			label.setText("Sort by");
			
			combo = new Combo(parent, SWT.DROP_DOWN);
			
			fillCombo();
		}		
	}
	
	/**
	 * 
	 */
	private void fillCombo() {
		for(IBulkEditorSortProvider sortProvider : sortProviders){
			Set<String> names = sortProvider.getComparatorNames();
			for(String name : names){
				combo.add(name);
				comparators.add(sortProvider.getComparatorByName(name));
			}
		}
		
		combo.addSelectionListener(new SelectionListener());
		combo.select(selectedIndex);
	}
	
	private class SelectionListener extends SelectionAdapter{
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			selectedIndex = combo.getSelectionIndex();
		}
	}
	
	/**
	 * 
	 */
	public Comparator getSelection() {
		return comparators.get(selectedIndex);
	}
}
