/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Assert;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInputType;
import eu.etaxonomy.taxeditor.bulkeditor.internal.TaxeditorBulkeditorPlugin;
import eu.etaxonomy.taxeditor.model.AbstractUtility;

/**
 * <p>BulkEditorUtil class.</p>
 *
 * @author n.hoffmann
 * @created Mar 12, 2010
 * @version 1.0
 */
public class BulkEditorUtil extends AbstractUtility{
	
	/**
	 * Opens a new editor window with the given input
	 * 
	 * @param input
	 * @param editorId
	 * @return 
	 * @throws PartInitException
	 */
	private static IEditorPart open(IEditorInput input, String editorId)
			throws PartInitException {
		return getActivePage().openEditor(input, editorId);
	}
	
	/**
	 * <p>openEmptyEditor</p>
	 *
	 * @param inputType a {@link eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInputTypeValues.BulkEditorInputType} object.
	 * @throws org.eclipse.ui.PartInitException if any.
	 */
	public static void openEmptyEditor(BulkEditorInputType inputType) throws PartInitException{
		AbstractBulkEditorInput editorInput = AbstractBulkEditorInput.NewInstance(inputType);
		open(editorInput, BulkEditor.ID);
	}
	
	/**
	 * <p>openEditor</p>
	 *
	 * @param entity a {@link eu.etaxonomy.cdm.model.common.IdentifiableEntity} object.
	 * @throws org.eclipse.ui.PartInitException if any.
	 */
	public static void openEditor(IdentifiableEntity entity) throws PartInitException{
		Assert.isNotNull(entity, "Entity must not be null");
		AbstractBulkEditorInput editorInput = AbstractBulkEditorInput.NewInstance(entity);
		open(editorInput, BulkEditor.ID);
	}
	
	/**
	 * <p>getPluginId</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public static String getPluginId(){
		return TaxeditorBulkeditorPlugin.PLUGIN_ID;
	}
	
	/**
	 * <p>getUndoContext</p>
	 *
	 * @return a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 */
	public static IUndoContext getUndoContext(){
		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
	}
}
