/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.command;

import org.apache.log4j.Logger;
import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.ui.IEditorInput;

import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.GroupEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.MediaEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.OccurrenceEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.TaxonEditorInput;

/**
 * @author p.ciardelli
 * @created 21.09.2009
 * @version 1.0
 */
public class BulkEditorPropertyTester extends PropertyTester {

    @SuppressWarnings("unused")
    private static final Logger logger = Logger
			.getLogger(BulkEditorPropertyTester.class);

	private static final String IS_MERGING_ENABLED = "isMergingEnabled";

	public static final String IS_GROUP_EDITOR = "isGroupEditor";
    public static final String IS_DERIVED_UNIT_EDITOR = "isDerivedUnitEditor";
    public static final String IS_TAXON_EDITOR = "isTaxonEditor";
    public static final String IS_CONVERT_ENABLED = "isConvertingEnabled";
    public static final String IS_PERSON = "isPerson";
    public static final String IS_TEAM = "isTeam";
    public static final String IS_MEDIA_EDITOR = "isMediaEditor";

    /** {@inheritDoc} */
	@Override
    public boolean test(Object receiver, String property, Object[] args,
			Object expectedValue) {
		BulkEditor bulkEditor = null;

		bulkEditor = (BulkEditor) receiver;
		if (IS_MERGING_ENABLED.equals(property)) {
			IEditorInput input = bulkEditor.getEditorInput();
			if (input instanceof AbstractBulkEditorInput) {
				return ((AbstractBulkEditorInput<?>) input).isMergingEnabled();
			}
		}
		if (IS_CONVERT_ENABLED.equals(property)) {
			IEditorInput input = bulkEditor.getEditorInput();
			if (input instanceof AbstractBulkEditorInput) {
				return ((AbstractBulkEditorInput<?>) input).isConvertingEnabled();
			}
		}
		else if(IS_GROUP_EDITOR.equals(property)){
			return bulkEditor.getEditorInput() instanceof GroupEditorInput;
		}
		else if (IS_DERIVED_UNIT_EDITOR.equals(property)){
            return bulkEditor.getEditorInput() instanceof OccurrenceEditorInput;
		}
        else if(IS_TAXON_EDITOR.equals(property)){
            return bulkEditor.getEditorInput() instanceof TaxonEditorInput;
        }
        else if(IS_MEDIA_EDITOR.equals(property)){
            return bulkEditor.getEditorInput() instanceof MediaEditorInput;
        }
		return false;
	}
}
