/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.command;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;
import org.eclipse.ui.menus.CommandContributionItem;
import org.eclipse.ui.menus.CommandContributionItemParameter;

import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorConstants;
import eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInputType;

/**
 * <p>DeleteObjectContributionItem class.</p>
 *
 * @author p.ciardelli
 * @created 25.08.2009
 * @version 1.0
 */
public class DeleteObjectContributionItem extends CompoundContributionItem {
	private static final Logger logger = Logger
			.getLogger(DeleteObjectContributionItem.class);

	/**
	 * <p>Constructor for DeleteObjectContributionItem.</p>
	 */
	public DeleteObjectContributionItem() {}

	/**
	 * <p>Constructor for DeleteObjectContributionItem.</p>
	 *
	 * @param id a {@link java.lang.String} object.
	 */
	public DeleteObjectContributionItem(String id) {
		super(id);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
	 */
	/** {@inheritDoc} */
	@Override
	protected IContributionItem[] getContributionItems() {
		
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		
		CommandContributionItemParameter parameter = 
			new CommandContributionItemParameter(
					window, null, IBulkEditorConstants.DYNAMIC_DELETE_OBJECT_ID, SWT.NONE);
		
		parameter.label = "Delete " + getEditorInputLabel(window) + " in Current Line";		
				
		return new IContributionItem[] {
				new CommandContributionItem(parameter)
		};
	}

	/**
	 * @param window
	 * @return
	 */
	private String getEditorInputLabel(IWorkbenchWindow window) {
		IEditorInput input = window.getActivePage().getActiveEditor().getEditorInput();
		
		BulkEditorInputType inputType = BulkEditorInputType.getByInput(input);
		return (inputType != null) ? inputType.label : "";
	}
}
