/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.texteditor.IDocumentProvider;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.strategy.merge.MergeException;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotation;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 12.05.2015
 *
 */
public class ConvertTeam2PersonHandler extends AbstractHandler {

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Person execute(ExecutionEvent event) throws ExecutionException {
		ISelection selection = HandlerUtil.getCurrentSelection(event);

		IEditorPart editor = HandlerUtil.getActiveEditor(event);

		IEditorInput input = editor.getEditorInput();
		if (editor.isDirty()){
			boolean proceed = MessageDialog.openQuestion(null,
					"Save changes", "You have made changes that must be saved before this query can be executed. Would you like to proceed?");
			if (proceed) {
				editor.doSave(null);
			} else {
				return null;
			}
		}
		if((input instanceof IEntityPersistenceService) && (selection instanceof IStructuredSelection)){


			IDocumentProvider provider = ((BulkEditor) editor).getDocumentProvider();
			LineAnnotationModel model =
					(LineAnnotationModel) provider.getAnnotationModel(input);


			IStructuredSelection structuredSelection = (IStructuredSelection) selection;

			IEntityPersistenceService persistenceService = (IEntityPersistenceService) input;

			Person person ;
			for(Object object : structuredSelection.toList()){
				LineAnnotation annotation = (LineAnnotation) model.getAnnotation(object);
				person = null;
				UpdateResult result = null;
				if (object instanceof Team){
					Team team = HibernateProxyHelper.deproxy(object, Team.class);
					try {
						result = CdmStore.getService(IAgentService.class).convertTeam2Person(team.getUuid());
					} catch (IllegalArgumentException e) {
						MessagingUtils.informationDialog("Can not convert Team to Person", e.getMessage());
					} catch (MergeException e) {
						MessagingUtils.informationDialog("Can not convert Team to Person", e.getMessage());
					}
				}else{
					MessagingUtils.informationDialog("Can not convert Team to Person", "convertTeam2Person can only be called on a team.");
				}
				if (result != null && result.isOk()){
				    person = (Person)result.getCdmEntity();
				}
				if (person != null){
					((BulkEditor) editor).removeAnnotatedLine(annotation);
					((BulkEditor) editor).createAnnotatedLine(person);
				}
			}
		}
		return null;
	}

}
