/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.handler;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.IAnnotatableEntity;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotation;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorUtil;
import eu.etaxonomy.taxeditor.bulkeditor.operation.SetMarkerFlagOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;

/**
 * <p>SetMarkerFlagTrueHandler class.</p>
 *
 * @author p.ciardelli
 * @created 20.10.2009
 * @version 1.0
 */
public class SetMarkerFlagHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		ISelection selection = HandlerUtil.getCurrentSelection(event);
		
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		
		Object[] data = (Object[]) ((Event) event.getTrigger()).data;
		
		if (selection instanceof TextSelection 
					&& editor instanceof BulkEditor) {
			TextSelection textSelection = (TextSelection) selection;
			BulkEditor bulkEditor = (BulkEditor) editor;
			
			LineAnnotationModel model = (LineAnnotationModel) bulkEditor.getDocumentProvider().getAnnotationModel(editor.getEditorInput());
			if (model != null) {
				Iterator iter = model.getAnnotationIterator(textSelection.getOffset(), textSelection.getLength(), true, true);
				
				Set<IAnnotatableEntity> annotatableEntities = new HashSet<IAnnotatableEntity>();
				while (iter.hasNext()) {
					Object next = iter.next();
					if (next instanceof LineAnnotation) {
						Object entity = ((LineAnnotation) next).getEntity();
						
						if(entity instanceof IAnnotatableEntity){
							annotatableEntities.add((IAnnotatableEntity) entity);
						}
					}
				}
								
				AbstractPostOperation operation = new SetMarkerFlagOperation("Set Marker", BulkEditorUtil.getUndoContext(), annotatableEntities, (MarkerType) data[0], (Boolean) data[1], bulkEditor);
				BulkEditorUtil.executeOperation(operation);
			}
		}
		return null;
	}
}
