/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.texteditor.IDocumentProvider;

import eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineEditor;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorConstants;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;

/**
 * <p>SetMergeTargetHandler class.</p>
 *
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class SetMergeTargetHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.AbstractHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	public Object execute(ExecutionEvent event) throws ExecutionException {
				
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		IEditorInput input = editor.getEditorInput();
		
		if (editor instanceof AnnotatedLineEditor && input instanceof AbstractBulkEditorInput) {
			
			IDocumentProvider provider = ((AnnotatedLineEditor) editor).getDocumentProvider();
			LineAnnotationModel model = 
					(LineAnnotationModel) provider.getAnnotationModel(input);
			IDocument document = provider.getDocument(input);
			
			ISelection selection = HandlerUtil.getCurrentSelection(event);
			if (selection instanceof TextSelection) {
				Annotation annotation = null;
				
				// Get annotation in current line
				int line = ((TextSelection) selection).getStartLine();
				annotation = model.getAnnotationAtLine(line, document);
				
				// Remove existing merge target
				model.removeTypeFromAllAnnotations(IBulkEditorConstants.TYPE_MERGE_TARGET);
				
				// Set annotation in current line to merge target
				if (annotation != null) {
					model.changeAnnotationType(
							annotation, IBulkEditorConstants.TYPE_MERGE_TARGET);
				}
			}
		}		
		return null;
	}
}
