/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.IGroupService;
import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.exception.ReferencedObjectUndeletableException;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator.GroupCreator;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Mar 9, 2011
 * @version 1.0
 */
public class GroupEditorInput extends AbstractBulkEditorInput<Group> {

	public static final String ID = "bulkeditor.input.group";

	private static GroupEditorInput instance;

	/**
	 * @return the instance
	 */
	public static GroupEditorInput getInstance() {
		if(instance == null){
			instance = new GroupEditorInput();
		}
		return instance;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	@Override
	public String getName() {
		return BulkEditorInputType.GROUP.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	@Override
	public String getToolTipText() {
		return getName();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#save(java.lang.Object)
	 */
	@Override
	public Group save(Group entity) {
	    return CdmStore.getService(IGroupService.class).merge(entity, true).getMergedEntity();
	    
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#delete(java.lang.Object)
	 */
	@Override
	public boolean delete(Group entity, DeleteConfiguratorBase config) throws ReferencedObjectUndeletableException {
		return CdmStore.getService(IGroupService.class).delete(entity.getUuid()) != null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#listEntities(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
	 */
	@Override
	protected List<Group> listEntities(
			IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.getSearchManager().findGroups(configurator);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#loadEntity(java.util.UUID)

	@Override
	protected Group loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{});
		return CdmStore.getService(IGroupService.class).load(entityUuid, propertyPaths);
	}
*/
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#createEntityCreator()
	 */
	@Override
	protected IEntityCreator<Group> createEntityCreator() {
		return new GroupCreator();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#getText(eu.etaxonomy.cdm.model.common.ICdmBase)
	 */
	@Override
	public String getText(Group entity) {
		return entity.getName();
	}

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.CdmEntitySessionInput#merge()
     */
    @Override
    public void merge() {

    }

	@Override
	protected Group loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{});
		return HibernateProxyHelper.deproxy(CdmStore.getService(IAgentService.class).load(entityUuid, propertyPaths), Group.class);
	}



}
