/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.IMediaService;
import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.MediaDeletionConfigurator;
import eu.etaxonomy.cdm.api.service.exception.ReferencedObjectUndeletableException;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.persistence.dao.hibernate.media.MediaDaoHibernateImpl;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorSortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator.MediaCreator;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.IdentifiableEntitySortProvider;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 10.03.2016
 *
 */
public class MediaEditorInput extends AbstractBulkEditorInput<Media> {
    public static final String ID = "bulkeditor.input.media";

    private static MediaEditorInput instance;

    public static String getID() {
        return ID;
    }

    public static AbstractBulkEditorInput getInstance() {
        if (instance == null) {
            instance = new MediaEditorInput();
        }
        return instance;
    }

    @Override
    public String getName() {
        return BulkEditorInputType.MEDIA.label;
    }

    @Override
    public String getToolTipText() {
        return getName();
    }

    @Override
    public Media save(Media entity) {
       return CdmStore.getService(IMediaService.class).merge(entity, true).getMergedEntity();
       
    }

    @Override
    public boolean delete(Media entity, DeleteConfiguratorBase config) throws ReferencedObjectUndeletableException {
    	MediaDeletionConfigurator mediaConfig = null;
    	if (config instanceof MediaDeletionConfigurator){
    		mediaConfig = (MediaDeletionConfigurator) config;
    	} else{
    		
    	}
        return CdmStore.getService(IMediaService.class).delete(entity.getUuid(), mediaConfig) != null;
    }

    @Override
    protected List<Media> listEntities(IIdentifiableEntityServiceConfigurator configurator) {
        return CdmStore.getSearchManager().findMedia(configurator);
    }

    @Override
    protected IEntityCreator<Media> createEntityCreator() {
        return new MediaCreator();
    }

    @Override
    public void merge() {
        // TODO Auto-generated method stub

    }

    @Override
    public List<IBulkEditorSortProvider<Media>> getSortProviders() {
        List<IBulkEditorSortProvider<Media>> sortProviders = super.getSortProviders();

        sortProviders.add(0, new IdentifiableEntitySortProvider<Media>());

        return sortProviders;
    }
    
	@Override
	protected Media loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{});
		return CdmStore.getService(IMediaService.class).load(entityUuid, propertyPaths);
	}

}
