/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.NameDeletionConfigurator;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorSortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator.NameCreator;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.IdentifiableEntitySortProvider;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>NameEditorInput class.</p>
 *
 * @author p.ciardelli
 * @created 20.08.2009
 * @version 1.0
 */
public class NameEditorInput extends AbstractBulkEditorInput<TaxonNameBase> {

	/**
	 *
	 */
	private static final long serialVersionUID = -3085029575759626823L;
	private static NameEditorInput instance;

	/** Constant <code>ID="bulkeditor.input.name"</code> */
	public static final String ID = "bulkeditor.input.name";

	/**
	 * <p>Getter for the field <code>instance</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput} object.
	 */
	public static AbstractBulkEditorInput getInstance() {
		if (instance == null) {
			instance = new NameEditorInput();
		}
		return instance;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	/**
	 * <p>getName</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getName() {
		return BulkEditorInputType.NAME.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	/**
	 * <p>getToolTipText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getToolTipText() {
		return getName();
	}

	/**
	 * <p>getID</p>
	 *
	 * @return a {@link java.lang.Object} object.
	 */
	public static Object getID() {
		return ID;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput#isMergingEnabled()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean isMergingEnabled() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public List listEntities(IIdentifiableEntityServiceConfigurator configurator) {
		
		configurator.setPropertyPaths(getPropertyPaths());
		return CdmStore.getSearchManager().findNames(configurator, this.getConversation());
	}

	/** {@inheritDoc}
	@Override
	public TaxonNameBase loadEntity(UUID uuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{});
		return CdmStore.getService(INameService.class).load(uuid, propertyPaths);
	}
*/
	@Override
    public boolean delete(TaxonNameBase entity, DeleteConfiguratorBase config)  {
		NameDeletionConfigurator nameConfig = null;
		if (config instanceof NameDeletionConfigurator){
			nameConfig = (NameDeletionConfigurator) config;
		} else{
			
		}
		return CdmStore.getService(INameService.class).delete(entity.getUuid(), nameConfig) != null;
	}
	
	/** {@inheritDoc} */
	@Override
    public TaxonNameBase save(TaxonNameBase entity) {
		TaxonNameBase mergedName = null;
		mergedName =  CdmStore.getService(INameService.class).merge(entity, true).getMergedEntity();
	   	return mergedName;
		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#createEntityCreator()
	 */
	@Override
	protected IEntityCreator<TaxonNameBase> createEntityCreator() {
		return new NameCreator();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#getSortProviders()
	 */
	@Override
	public List<IBulkEditorSortProvider<TaxonNameBase>> getSortProviders() {
		List<IBulkEditorSortProvider<TaxonNameBase>> sortProviders = super.getSortProviders();

		sortProviders.add(0, new IdentifiableEntitySortProvider<TaxonNameBase>());

		return sortProviders;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#getTypeText(java.lang.Object)
	 */
	@Override
	public String getTypeText(Object entity) {
		if((entity instanceof TaxonNameBase) && (((TaxonNameBase) entity).getNomenclaturalCode() != null)){
			return ((TaxonNameBase) entity).getNomenclaturalCode().getTitleCache();
		}
		return super.getTypeText(entity);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#getText(eu.etaxonomy.cdm.model.common.ICdmBase)
	 */
	@Override
	public String getText(TaxonNameBase entity) {
		return (entity).getFullTitleCache();
	}

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.CdmEntitySessionInput#merge()
     */
    @Override
    public void merge() {

    }

	@Override
	protected TaxonNameBase loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{});
		return CdmStore.getService(INameService.class).load(entityUuid, getPropertyPaths());
	}


    private List<String> getPropertyPaths(){
    	 List<String> nameBasePropertyPaths = Arrays.asList(new String[] {
    	 "name.descriptions",
         "name.typeDesignations",
         "name.status",
         "name.nomenclaturalReference.inReference",
         "name.taxonBases.taxonNodes",
         "name.relationsFromThisName",
         "name.relationsToThisName",
         "name.homotypicalGroup.typifiedNames.taxonBases.synonymRelations.synonym.name.status",
         "name.homotypicalGroup.typifiedNames.relationsToThisName.fromName"});
    	 
    	 return nameBasePropertyPaths;
    }

}
