/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator;

import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.WizardDialog;

import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorUtil;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.newWizard.NewDerivedUnitBaseWizard;

/**
 * <p>
 * OccurrenceCreator class.
 * </p>
 *
 * @author n.hoffmann
 * @created Jun 16, 2010
 * @version 1.0
 */
public class OccurrenceCreator implements
		IEntityCreator<SpecimenOrObservationBase> {
	private static final Logger logger = Logger
			.getLogger(OccurrenceCreator.class);

	/** {@inheritDoc} */
	@Override
	public SpecimenOrObservationBase createEntity(String text) {

		return createEntity(DerivedUnit.class, text);
	}

	/** {@inheritDoc} */
	@Override
	public SpecimenOrObservationBase createEntity(Object key, String text) {

		SpecimenOrObservationBase specimenOrObservation = null;
		NewDerivedUnitBaseWizard wizard = new NewDerivedUnitBaseWizard((SpecimenOrObservationType)key);
		wizard.init(null, null);
		if(wizard.getEntity() != null) {
			WizardDialog dialog = new WizardDialog(BulkEditorUtil.getShell(), wizard);
			int status = dialog.open();

			if (status == IStatus.OK) {
				specimenOrObservation = wizard.getEntity();
			}
		}
		return specimenOrObservation;
	}

	/**
	 * <p>
	 * getKeyLabelPairs
	 * </p>
	 *
	 * @return a {@link java.util.Map} object.
	 */
	@Override
	public Map<Object, String> getKeyLabelPairs() {
	    Map<Object, String> keyLabelPairs = new LinkedHashMap<Object, String>();
	    keyLabelPairs.putAll(AbstractUtility.orderTerms(Arrays.asList(SpecimenOrObservationType.values())));
	    return keyLabelPairs;
	}

	@Override
	public boolean savesEntity() {
		return true;
	}


}
